

;**************************************************************************
;                      Micro908 "Antenna Analyst"
;                              AA-908
;
;                        George Heron, N2APB
;                        2419 Feather Mae Ct.
;                        Forest Hill, MD  21050
;                        email: n2apb@amsat.org
;
; -------------------------------
; Contributors to the sourcecode:
;
;   Link908 & data sending:
;       Michael L. Hasenfratz, WA6FXT
;       131 Columbia Rd.
;       Thousand Oaks, Ca. 91360
;       email: wa6fxt@arrl.net
;
;   SEEPROM Driver & integration with AF908
;       Steve Holton, N1NB
;       9 Alewives Road
;       Norwalk, CT 06850
;       email: sholton@optonline.net
;
;---------------------------------
;
; Revision History:
; ----------------
; 1.0      10/4/2003      Initial prototype release before Pacificon QRP Forum
; 1.1      1/18/2004      Fix for dial granularity
; 1.3      2/28/2004      (1) Added better use of dial for submenu selection, (2) Fixed display intermittency
; 1.4      3/31/2004      (1) Added SWR calculations in Manual mode, (2) Added Check4Min SWR during
;                         Scan, (3) Changed stack to $03FF, (4) Added Custom Settings: Start, End, Step;
;                         (5) Added EEPROM save/restore for user settings, (6) Changed Scan end-checking
;                         to use Scan_End variable, (7) Added "View User Settings" to Config menu
; 1.5      4/14/2004      (1) New way to do piezo tones, (2) Tied tones in with SWR when in Manual.
; 1.6      5/11/2004      User interface changes: (1) Continuous SWR & Z measurements in Manual,
;                         (2) User dumped into Manual at end of Scan, (3) Config renamed to Setup,
;                         (4) Menu messages trimmed & reordered, (5) Cal mode removed from Mode menu,
;                         (6) Dynamic menu entry for Setup, Band and Scan now avail from within Manual,
;                         (7) Resisance measurements now made, (8) SWR fixes yield right value for R>50.
; 1.7      5/13/2004      Resistance measurements cleaned up.
; 1.7e     6/04/2004      Added ability for HC908 Daughtercard to programmatically
;                         download/flash a new application with command from the
;                         application code, in cooperation with custom "m908
;                         Loader" program on PC communicating with HC908 card
;                         over serial link.
; 2.0      6/22/2004      (1) Implemented fourth set of reflectometer detectors and buffer amps for "Va" voltage
;                         in order to calculate better Z. Works great! (2) Added Debug Mode that sends intermediate
;                         computations to the serial terminal.
; 3.0k     9/02/2004      (1) Added Calibration, (2) enter to Set DDS limits at prior settings,
;                         (3) added 100 kHz step size, (4) used greater precision computations,
;                         (4) added busy LED during Save Settings, (5) added "exit=<Setup>" in Set Start/End,
;                         (5) added DEBUG mode to dump data block, (6) added "Return to Default" Config cmnd,
;                         (7) fixed Scan hanging bug (properly set Scan_End(0:7))
;                         (8) brought Debug Monitor command up to main level in Config menu,
;                         (9) eliminated Diags module (not needed)
; 3.0k6    10/18/2004     (1) Changed to using ERA-3 with different gain resistors.  Getting rid of MAV-11 solved
;                         high-end signal level rolloff
;                         (2) Changed to "back_to_basics" simplified calibration, adjusting Vr to make SWR=5.4
;                         with 270-ohm load.  Best performing results ever!!
;                         (3) Added LCD backlight control circuit (and software timers, Setup menus, etc.)
;                         (4) Fixed bug in calculating R (was improperly scaling large denominator)
; 4.0a    11/15/2004      Started measuring and adjusting for A/D offset voltages.  Improved accuracy a bit.
;                         Now handling remainder of correction factor generation and use.  Yields best results yet!
;                         Fixed cursor adjust too far to left.
;                         Straightened out Lite control and EEPROM save/restore
;                         Moved cal data to EEPROM Page 1.
;                         Fixed power-on pushbutton so timing not critical on release.
;                         Added "Stopped" display at powerdown.
;                         Made the Default Settings option more sensible and usable.
;                         Determine & display reactance sign
;                         Added modes for Inductance, Capacitance and VFO
; 4.0b    11/28/2004      Removed experimental code for Power and LCD Backlight Control.  (Caused probs in v4.0a)
;                         Removed Config "Software Update" option (used with m908 Loader pgm on PC) ... too error prone.
; 4.0c    12/05/2004      Mode pushbutton user interface change
;                         Calibration point changed to 10 MHz
; 4.1a    01/23/2005      Band selection increased to include WARC bands.  (WA6FXT)
;                         Added Link908 interface (WA6FXT)
; 4.1c    02/12/2005      Corrected stack bugs in Get_Dial_Selection and Get_YN routines
;                         Corrected portA bug in LCD driver
;                         Corrected Reactance Sign feature (in x_sign.asm)
;                         Merged WARC Band sppt and raw Link908 data transmit to mainline code
;                         Added Config options for scan data (no save, send to PC, send to SEEPROM)
;                         Added placeholders for SEEPROM code (search .LST file for "INSERT SEEPROM USAGE HERE")
; 4.1d    02/13/2005      Merged SEEPROM development with mainline code, with SEEPROM test function hooked to PB5
; 4.1e    02/17/2005      Added R & X send options for Link908 (WA6FXT)
;                         Corrected stack error in scan.asm (send_n_mem:) (WA6FXT)
;                         Corrected bug in link.asm (CustomBand) (WA6FXT)
; 4.1     02/27/2005      Production code = supports reactance sign and Link908
; 4.1e    03/03/2005      Temporarily disabled "jsr disp_sign" in ISR until "constant SWR>10" condition fixed
; 4.2     03/24/2005      Fixed "SWR>10" condition, associated with reactance sign feature for narrow resonance readings.
;                         (Properly observed No_Disp_Flag in Calc.asm for clamp condition during diddle (see clamp:))
;                         Ensures max setting of 30 MHz for specifying End Freq in Config
; 4.2a    03/28/2005	  Found another way to invoke entry to HCmon for the Software Load function in Config, thus
;			  eliminating the occasional halted downloading and need for using the "P3 jumper method" of loading.
;			  Corrected the upper band limit for 12m operation.
; 4.2b    03/30/2005      Fixed tone to properly swing to min freq during Scan ... and stay there when scan complete.
;                         Display reactance sign only when Sleep PB is pressed.  This may help create more stable X readings.
;                         Added loop to Config selection of Battery display (allows monitoring batt voltage during charge).
;                         Removed a lot of unnecessary code and comments from the SEEPROM testing days.
; 4.3a    04/23/2005      Added LCD Backlight control.  (Optionally uses different LCD and new transistor to board.)
;                         Restructured the source code to provide integration with AF908 module. (Compile time switches.)
;
;**************************************************************************
; Copyright 2005 by G. Heron, N2APB.  All rights reserved.
;**************************************************************************

         include "ramdefs.asm"	      ;module defines all RAM and port usage

;**************************************************************************
;**********************      USER APP ENTRY POINT     *********************
;**************************************************************************
;        This is the point ($8040) where User Code starts executing       *
;        after coming from the Monitor "X" command (eXecute User App)     *
;        or after board gets a RESET (when Monitor jumper is off)         *
;**************************************************************************

        org     $8040

Start:
        sei                            ;disable all interrupts
$IF FULL_DEBUGGER                      ; if true then full debugger capability
        ldhx    #$01ff
$ELSEIF
        ldhx    #$03FF                 ;initialize
$ENDIF

        txs                            ;  the stack pointer

Restart:
        jsr     Init_all               ;initialize system variables and devices

        cli                            ;allow interrupts to happen

;**************************************************************************
;**************************************************************************
;*****************      USER APPPLICATION MAINLINE      *******************
;**************************************************************************
;**************************************************************************

User_Main:
        mov     #3,timeout1            ;I'm not sure why this is needed (but it is!)
        jsr     LCD_init
        sta     copctl                 ;clear the COP counter
        lda     #100
        jsr     delay

             ;display battery voltage
        jsr     LCD_Clear
        ldhx    #Batt_msg               ;"Battery Voltage:"
        jsr     Disp_Msg
        jsr     Calc_Batt
        lda     #$89                    ;point to mid-1st line
        jsr     Cmnd2LCD
        jsr     dispb2                  ;display battery voltage
        jsr     wait500ms
        clra
        sta    manualDL                 ; set manual download flag off in SPI file XFER
$IF AA_AF_Both
        sta     MSG_instant             ; set instant process dial flag off
        sta     MSG_one                 ; sett single command flag off
$ENDIF

startmain:
;        tone_off                        ;turn tone off while in Main menu
st7:    jsr     LCD_Clear
        ldhx    #AA1_msg
st5:    jsr     disp_msg                ;display banner on 1st line
        jsr     wait500ms
        jmp     Impedance               ;go right into Impedance Mode (in Mode.asm module)

AA1_msg: fcb     '  AA908 4.3a',0
AA2_msg: fcb     'Select Function',0


;**************************************************************
;  INCLUDED MODULES
;**************************************************************

      include "mode.asm"
      include "config_m.asm"
      include "calib.asm'
      include "scan.asm"
      include "bands.asm"
      include "timer.asm"
      include "drivers.asm"
      include "misc.asm"
      include "math.asm"
      include "debug.asm"
      include "calcs.asm"
      include "init_m.asm"
      include "x_sign.asm"
      include "link.asm"
      include "SPI_functions.asm"    ; SEEPROM functions  SMH
      include "SPI_FileXfer.asm"     ; SEEPROM<->PC data transfer functions SMH





