;*********************************************************************************
;*********************************************************************************
;                          DSP Run and DSP Cmd Functions
;     Entered from PB1_Run in PB1.asm in response to user selecting "Run"
;     validates SEEPROM to be sure DSP prorgams are present
;     DSP_Run: loads program and starts it.
;     Also contains the Run/PC (DSP_PCcmd) & routines and utility functions for
;     resetting the DSP, and managing SEEPROM direcotry etc
;
;     written January 2005 by Steve Holton N1NB
;*********************************************************************************
;*********************************************************************************

;-------------DSP image format
PGMCOUNT   equ 4   ;* offset in file of program count */
TOTALBYTES equ 5   ;* offset in file of total byte count (excl. checksum) */
DIRSTART   equ 7   ;* offset to first directory entry
DIRSIZE    equ 12  ;* length of directory  entry */
PGMNAME    equ 0   ;* offset of pgm name in dir. entry
NAMELEN    equ 7   ;* MAX length of name (less teminating null) */
OFFSET     equ 8   ;* offset within directory entry of byte offset of pgm start */
BYTECNT    equ 10  ;* offset within directory entry of bytecount */

;* File structure */
;* offset 0   DSP signature "%dSp"
;* offset 4   Program count  # of DSP programs in this image
;* offset 5   Total byte count nnnn in file (excluding checksum)
;* offset 7  1st of up to 10 directory entries for DSP programs
;* Offset within Directory entry (12 bytes long)
;* offset 0   Program name  Max 7 characters + null
;* offset 8   Offset within image of start for this program
;* offset 10  total number of bytes for this program
;*
;*    following last directory entry the DSP programs
;*       the form of the program images is a series of DSP download commands
;*       and the data in the exact form required by the DSP monitor's loader
;*       Each command and it's data are less than 256 bytes so that each
;*       command entirely fits in the SEEPROM buffer in RAM
;*       the commands used are:
;*           $DPxxyy.....Program memory data 24 bit words
;*           $PMxxzz.....initialized Data memory 16 bit words
;*               xx= load address
;*               yy= 24 bit word count
;*               zz= 16 bit word count
;*
;* offset nnnn-4 DSP signature "%dSp"
;* offset nnnn  24 bit check sum on entire file



DSP_run:
        jsr     findDSP                 ; locate start of DSP programs in SEEPROM
        lda     DSPdefault_load         ; check if we auto load default pgm
        beq     need_pgm                ; no we must go get it from user
        psha                            ; if yes save value
        clra
        sta     DSPdefault_load         ; set to 0 so only done at startup
        jsr     DSPreadDirChkSig        ; yes validate there is such a program
        pula                            ; retrieve value (keep stack honest too)
        bcs     D_r_Fail                ; no DSP programs in SEEPROM say so
        cmp     {SPI_buffer1+PGMCOUNT}  ; see if there are enough pgms in directory
        bls     run_nameChk             ; yes now check name
        jmp     RunFailAuto2            ; if not this will ignore and go to select pgm
D_r_Fail:
        jmp     RunFailAuto1
run_nameChk:
        sub     #1                      ; value-1 is DIR entry # to pgm to load
        psha
        jsr     getDIRentry
        sthx    from_addr
        ldhx    #DSPdefault_name
        sta     to_addr
        jsr     StringCompare           ; name check confirmation
        pula
        beq     RunFailAuto2            ; if not this will ignore and go to select pgm
        bra     run_direct
need_pgm:
        jsr     DSPreadDirChkSig        ; see if there is only one program
        bcs     RunFailAuto1            ; if no programs say so
        lda     #1
        cmp     {SPI_buffer1+PGMCOUNT}
        bne     get_pgm                 ; more than one have to ask user
        sub     #1                      ; only one program so index=0
        bra     run_direct              ; so we can load it directly
get_pgm:
        jsr     DSP_pgm_select
        bcs     ExitRun
run_direct:
        psha                            ; save pgm index
        jsr     LCD_Clear
        ldhx    #loading_msg            ; put loading DSP msg up while loading
        jsr     disp_msg
        lda     #$C0
        jsr     Cmnd2LCD                ;write to 2nd line next
        ldhx    #loading_msg2           ; put loading DSP msg part 2 up
        jsr     disp_msg
        lda     1,sp                    ; get copy of index
        jsr     getDIRentry             ; get h:x pointing to Dir entry
        aix     #PGMNAME                ; yes this is zero now but for future safety
        jsr     disp_msg                ; put name of pgm on LCD
        pula                            ; retrieve index again
        jsr     DSPload
        bne     RunFail
        jsr     DSPGo
        bne     RunFail
        brclr   ENC_pb,portF,*      ;wait here until Encoder PB released

ExitRun:
; print end msg
        ldhx     #DSPmsg2
        jsr      _puts
        jsr      _crlf
        clra                        ; say success
        rts

RunFail:
        jsr     LCD_Clear
        ldhx    #DSP_RunFail       ; tell the world
        jsr     disp_msg
        jsr     wait500ms          ; let them see it
        jsr     wait500ms
        lda     #1                 ; say failed
        rts

RunFailAuto1:                      ; No DSP pgms in SEEPROM
        jsr     LCD_Clear
        ldhx    #Select_nopgmMSG   ; tell the world
        jsr     disp_msg
        lda     #$C0
        jsr     Cmnd2LCD           ;write to 2nd line next
        ldhx    #Select_nopgmMSG1  ; tell the world
        bra     R_F_A

RunFailAuto2:                      ; Default load DSP program not in SEEPROM
        jsr     LCD_Clear
        ldhx    #DSP_RunFail2      ; tell the world
        jsr     disp_msg
        lda     #$C0
        jsr     Cmnd2LCD           ;write to 2nd line next
        ldhx    #DSP_RunFail3     ; tell the world
R_F_A:  jsr     disp_msg
        jsr     wait500ms          ; let them see it
        jsr     wait500ms
        jsr     wait500ms
        lda     #1                 ; say failed
        rts


DSP_sig:  fcb   "%dSp",0
DSP_RunFail: fcb  'DSP Run Failed',0
DSP_RunFail2: fcb  'AutoLoad Failed',0
DSP_RunFail3: fcb  'DSP PGM Missing',0
loading_msg: fcb 'Loading DSP with',0
loading_msg2: fcb 'Pgm=> ',0
Select_nopgmMSG: fcb  'SRI No DSP pgms',0
Select_nopgmMSG1: fcb  'D/L DSP pgm 1st',0
;--------------------------


DSPload:
        psha                       ; save program number
        ldhx     #DSPmsg1          ; print start message on serial port
        jsr      _puts

        jsr      wait100ms         ; let the serial port drain to be safe

; Set DSP port direction

        bclr     DSP_TXD,ddrA      ; set DSP's tXd as input to us
        bset     DSP_RXD,ddrA      ; set DSP's RxD as output from us
        bset     DSP_RST,ddrA      ; set DSP's reset as output from us
        bset     DSP_RST,portA     ; set /RST High

        jsr      DSPreset_banner    ; Reset DSP

        jsr      wait100ms
        jsr      wait32ms
        ldhx     #spi_buffer1
        jsr      SendOKget_ok             ; send alive msg recv result
        lda      #'o'
        cmp      spi_buffer1
        bne      DSPnotOK
        lda      #'k'
        cmp      spi_buffer1+1
        beq      DSPalive
DSPnotOK:
        ldhx     #DSP_aliveNotOKmsg     ; sometimes we get here but its
        jsr      _puts                  ; always really OK
        bra      tryDL                  ; so we carry on
DSPalive:
        ldhx     #DSP_aliveOKmsg
        jsr      _puts



tryDL:

        ldhx    #SPI_buffer1
        sthx    SPI_RamAddr
        lda     DSP_offset         ; start of DSP programs
        sta     SPI_offset         ; to offset for read directory
        lda     DSP_offset+1       ; start of DSP programs
        sta     SPI_offset+1       ; to offset for read directory
        lda     #101               ; read 106 bytes filecount plus 10 max entries
        ldhx    #SPI_Args
        jsr     SPI_Read

        pula                        ; recover pgm number
        jsr     getDIRentry         ; returns directory entry in H:X
        jsr     checkdsp908         ; check if name is dsp908 (H:X unchanged)
        jsr     checkASP            ; check if name is ASP (H:X unchanged)

DoDLandGO:
; with H:X pointing to the directory entry of desired program
        lda     BYTECNT,x
        sta     DSP_bytecount      ; save total bytecount this program
        lda     BYTECNT+1,x
        sta     DSP_bytecount+1
        lda     OFFSET+1,x         ; get offset in image
        add     DSP_Offset+1       ; plus offset to image
        sta     SPI_offset+1       ; equals offset in SEEPROM
        lda     OFFSET,x
        adc     DSP_Offset
        sta     SPI_offset         ; save offset of program start


loadPGMloop:
        ldhx    #SPI_buffer1
        sthx    SPI_RamAddr
        clra
        ldhx    #SPI_Args
        jsr     SPI_Read           ; read 256 bytes
        ldhx    #SPI_buffer1
        lda     #'P'               ; is this PM ($DP) command?
        cmp     2,x
        bne     DownloadDM         ; NO go do DM

; we KNOW that the total byte count of the command including the command
;   is less than or equal to 256 - because the utility has made it so on the PC side
;   hence the instr. count also less that 256
DownloadPM:
        lda     6,x                ; get low order byte of instruction count
        lsla                       ; times 2
        add     6,x                ; plus itself = times 3 (3 bytes/intsr.)
        add     #7                 ; plus command + start addr.+ instr. count
        psha                       ; put byte count on stack for loop counter
        sta     DSP_currentCnt+1   ; save in 16 bit form for total count
        cbeqa   #0,dl_is256
        clra
        sta     DSP_currentCnt     ; less than 256 clear hi byte
        bra     dl_ck
dl_is256:
        lda     #$01               ; make it true 256
        sta     DSP_currentCnt
dl_ck:  lda     6,x                ; get instruction count again
        aix     #7                 ; point H:X to actual instructions
        jsr     computeCksum24

        ldhx    #SPI_buffer1       ; point to start of command
dl_pm1: lda      ,x                ; send the command and data
        jsr      DSPputch
        aix      #1
        dec      1,sp
        bne      dl_pm1

        lda      #3                 ; read back 3 bytes of checksum
        sta      1,sp
        ldhx     #SPI_buffer1
dl_pm2:
        jsr      DSPgetchne
        sta      ,x
        aix      #1
        dec      1,sp
        bne      dl_pm2
        pula                       ; toss byte count used as counter for loops
        ldhx    #SPI_buffer1
        jsr     compareCksum24

DownloadCommon:
        bne     DL_error
        lda     DSP_bytecount+1   ; total bytecount
        sub     DSP_currentCnt+1  ; subtract original byte count for this cmd
        sta     DSP_bytecount+1
        lda     DSP_bytecount
        sbc     DSP_currentCnt
        sta     DSP_bytecount
        ora     DSP_bytecount+1   ; are we at zero = the end this pgm?
        beq     DLdone
        lda     DSP_currentCnt+1
        add     SPI_offset+1      ; & advance SEEPROM offset by bytecount
        sta     SPI_offset+1
        lda     DSP_currentCnt
        adc     SPI_offset
        sta     SPI_offset
        jmp     loadPGMloop
DL_error:
        ldhx    #DSP_DL_fail
        jsr     _puts
        jmp     fail_exit

DownloadDM:
        lda     6,x                ; get low order byte of data word count
        lsla                       ; times 2  (2 bytes per DM word)
        add     #7                 ; plus command + start addr.+ instr. count
        psha                       ; put byte count on stack for loop counter
        sta     DSP_currentCnt+1   ; save in 16 bit form for total count
        cbeqa   #0,dld_is256
        clra
        sta     DSP_currentCnt     ; less than 256 clear hi byte
        bra     dld_ck
dld_is256:
        lda     #$01               ; make it true 256
        sta     DSP_currentCnt
dld_ck
        lda     6,x                ; get data count again
        aix     #7                 ; point H:X to actual data
        jsr     computeCksum16     ; computes 16 bit check sum
        ldhx    #SPI_buffer1       ; point to start of command
dl_dm1: lda     ,x                 ; send the command and data
        jsr     DSPputch
        aix     #1
        dec     1,sp
        bne     dl_dm1

        lda     #2                  ; read back 2 bytes of checksum
        sta     1,sp
        ldhx    #SPI_buffer1
dl_dm2:
        jsr     DSPgetchne
        sta     ,x
        aix     #1
        dec     1,sp
        bne     dl_dm2
        pula                        ; toss byte count used as counter for loops
        ldhx    #SPI_buffer1
        jsr     compareCksum16
        bra     DownloadCommon

DLdone:
        ldhx    #DSP_DL_OK
        jsr     _puts
        clra
        rts

DSPgo: lda      #'$"            ; start it up
        jsr      DSPputch
        lda      #'G'
        jsr      DSPputch
        lda      #'O'
        jsr      DSPputch
        lda      #$00
        jsr      DSPputch
        lda      #$00
        jsr      DSPputch
        jsr      DSPgetchne
        cmp      #$00
        bne      go_fail
        jsr      DSPgetchne
        cmp      #$00
        beq      go_ok
go_fail:
        ldhx     #DSP_NoGo
        jsr      _puts
        bra      Fail_Exit
go_ok:
        ldhx     #DSP_GoOK
        jsr      _puts
        clra
        rts

Fail_exit:
        lda    #1
        rts


DSPreset_banner:
; Reset DSP
        bclr     DSP_RST,portA     ; bring /rst low
        jsr      wait100ms
        bset     DSP_RST,portA     ; bring /rst hi again
        ldhx     #632

findstart:
        jsr     DSPgetchne
        cmp     #'!'
        bne     findstart
CharPassLoop:
        jsr      DSPgetchne
        aix      #-1
        cmphx    #$0000
        bne      CharPassLoop
        rts
;---------- run DSP program ustilizing PC serial terminal to control
;             two special cases dsp908 and ASP 9n these cases we know the
;             characterstics of their interaction with PC so we can
;             handle with the software UART and the SCI UART
;             for all others we enter bypass mode - without a reset and let
;             running PC and DSP program interact directly until bypass mode
;             is exited
DSP_PCcmd:
        jsr     LCD_Clear       ; Show Select DSP action and end DSP pgm
        ldhx    #PB1_msg
        jsr     disp_msg
        lda     #$C0
        jsr     Cmnd2LCD                ;write to 2nd line next
        ldhx    #simpleMsgTable
        jsr     disp_msg
        lda     DSPisdsp908
        bne     DSPcmda         ; if dsp908
        lda     DSPisASP
        beq     DSPcmdd         ; if not ASP
        ldhx    #ASPprompt      ; if ASP start new line and put out
        jsr     _puts           ;  1st prompt as we won't get to direct in time
DSPcmdd:
        lda     #0
        sta     PB5_reset
        jsr     DSP_Direct      ; All others
        rts
ASPprompt: fcb 'ASP running',CR,LF,'=>',0
dsp908prompt: fcb 'dsp908 running',CR,LF,0
DSPcmda:
        ldhx    #dsp908prompt       ; if dsp908 announce it
        jsr     _puts
DSPcmdloop:
        brclr   ENC_pb,portF,cmdbreak    ; check for Encoder PB pushed
        brclr SCRF,scs1,DSPcmdloop    ;SCRF = 1 ?
        jsr     _getch                ; wait for character from PC
        cmp     #'q'                  ; look for quit
        beq     cmdbreak
        cmp     #'$'            ; if $ then 2 byte command 4 hex in ascci
        bne     is1bytecmd
        lda     #4              ; will follow
        ldhx    #SPI_buffer1
        psha
get4:   jsr     _getch
        sta     ,x
        aix     #1
        dec     1,sp
        bne     get4
        pula
        lda     SPI_buffer1
        psha
        pulh
        ldx     SPI_buffer1+1
        jsr     _ahex2b
        ora     #$80             ; make sure it's seen as 2 byte command
        jsr     DSPputch
        lda     SPI_buffer1+2
        psha
        pulh
        ldx     SPI_buffer1+3
        jsr     _ahex2b
is1bytecmd:
        jsr     DSPputch              ; send to DSP
        jsr     DSPgetchne            ; get reply
        sta     SPI_buffer1
        jsr     DSPgetchne
        sta     SPI_buffer1+1
        and     SPI_buffer1
        cmp     #$ff
        beq     cmderr
        ldhx    #DSPcmdok
        jsr     _puts              ; send to PC
        lda     SPI_buffer1
        jsr     _pbyte
        ldhx    #DSPfilter
        jsr     _puts              ; send to PC
        lda     SPI_buffer1+1
        jsr     _pbyte
        jsr     _crlf
        bra     DSPcmdloop
cmderr:
        ldhx    #DSPcmderr
        jsr     _puts
        bra     DSPcmdloop
cmdbreak:
        jsr      DSPreset_banner    ; Reset DSP
        rts



DSPcmderr:  fcb  '->huh?',CR,LF,0
DSPcmdok:   fcb  '->OK status:$',0
DSPfilter:  fcb  ' filter info:$',0
sendOKget_ok:
        psha
        lda      #'$'
        jsr      DSPputch
        lda      #'O'
        jsr      DSPputch
        lda      #'K'
        jsr      DSPputch
        lda      #8
        psha
get8:   jsr      DSPgetchne         ; get back 'ok'
        sta      ,x
        aix      #1
        dec      1,sp
        bne      get8
        pula
        pula
        rts

getDIRentry:        ; compute directory offset of selected DSP pgm
                    ; index into directory in Acc
                    ; returns pointer to entry in h:x
        ldx     #DIRSIZE           ; directory size
        mul                        ; times = byte offset
        ldhx    #{SPI_buffer1+DIRSTART}  ; first directory entry
        pshh                       ; push h:x
        pshx
        add     1,sp               ; so we can add them to
        sta     1,sp               ; offset in A (which can't be >256)
        clra
        adc     2,sp
        sta     2,sp
        pulx                       ; recover updated offset to correct entry
        pulh
        rts

computeCksum24:       ; buffer ptr in h:x
                    ; number of 24bit words in A
                    ; 32 bit sum in intacc1
        psha
        clr      intacc1
        clr      intacc1+1
        clr      intacc1+2
        clr      intacc1+3
continueCksum24:                 ; buffer in h:x continue computing checksum
                                 ; with checksum already in intacc1
CSloop:
        lda     intacc1+3        ;process LS byte first
        add     2,x
        sta     intacc1+3
        bcc     CSua1
        inc     intacc1+2
        bne     CSua1
        inc     intacc1+1
        bne     CSua1
        inc     intacc1+0

CSua1:  lda     intacc1+2
        add     1,x
        sta     intacc1+2
        bcc     CSua2
        inc     intacc1+1
        bne     CSua2
        inc     intacc1+0

CSua2:  lda     intacc1+1
        add     ,x
        sta     intacc1+1
        bcc     CSua3
        inc     intacc1+0
CSua3:  aix     #3
        dec     1,sp
        bne     CSloop
;;;;;remove??        clr      intacc1   ; make it 24 bits
        pula
        rts
computeCksum16:       ; buffer ptr in h:x
                    ; number of 24bit words in A
                    ; 32 bit sum in intacc1
        psha
        clr      intacc1
        clr      intacc1+1
        clr      intacc1+2
        clr      intacc1+3
CSloopa:
        lda     intacc1+3        ;process LS byte first
        add     1,x
        sta     intacc1+3
        bcc     CSua1a
        inc     intacc1+2
        bne     CSua1a
        inc     intacc1+1
        bne     CSua1a
        inc     intacc1+0

CSua1a: lda     intacc1+2
        add     ,x
        sta     intacc1+2
        bcc     CSua2a
        inc     intacc1+1
        bne     CSua2a
        inc     intacc1+0

CSua2a  aix     #2
        dec     1,sp
        bne     CSloopa
        clr      intacc1+2   ; make it 16 bits
        pula
        rts

compareCksum24:             ; compares 24 bit check sum in h:X with one in intacc1
        lda     ,x          ; returns Z flag set if OK clear if bad
        cmp     intacc1+1
        bne     cmpBad
        aix     #1
compareCKsum16:
        lda     ,x
        cmp     intacc1+2
        bne     cmpBad
        aix     #1
        lda     ,x
        cmp     intacc1+3
cmpbad: rts

checkdsp908:                    ; Dir entry in H:X - leave in H:X
        lda     PGMNAME,x
        cmp     #$40                ; is letter?
        bls     AFname1             ; no number or special
        and     #$DF                ; force upper case
AFname1:cmp     DSP_dsp908_l
        bne     notAF
        lda     PGMNAME+1,x
        cmp     #$40                ; is letter?
        bls     AFname2             ; no number or special
        and     #$DF                ; force upper case
AFname2:cmp     DSP_dsp908_l+1
        bne     notAF
        lda     PGMNAME+2,x
        cmp     #$40                ; is letter?
        bls     AFname3             ; no number or special
        and     #$DF                ; force upper case
AFname3:cmp     DSP_dsp908_l+2
        bne     notAF
        lda     PGMNAME+3,x
        cmp     #$40                ; is letter?
        bls     AFname4             ; no number or special
        and     #$DF                ; force upper case
AFname4:cmp     DSP_dsp908_l+3
        bne     notAF
        lda     PGMNAME+4,x
        cmp     #$40                ; is letter?
        bls     AFname5             ; no number or special
        and     #$DF                ; force upper case
AFname5:cmp     DSP_dsp908_l+4
        bne     notAF
        lda     PGMNAME+5,x
        cmp     #$40                ; is letter?
        bls     AFname6             ; no number or special
        and     #$DF                ; force upper case
AFname6:cmp     DSP_dsp908_l+5
        bne     notAF
        lda     #1
        sta     DSPisdsp908
notAF:  rts


checkASP:
        lda     PGMNAME,x
        cmp     #$40                ; is letter?
        bls     ASPname1            ; no number or special
        and     #$DF                ; force upper case
ASPname1:cmp    DSP_ASP_l
        bne     notASP
        lda     PGMNAME+1,x
        cmp     #$40                ; is letter?
        bls     ASPname2            ; no number or special
        and     #$DF                ; force upper case
ASPname2:cmp    DSP_ASP_l+1
        bne     notASP
        lda     PGMNAME+2,x
        cmp     #$40                ; is letter?
        bls     ASPname3            ; no number or special
        and     #$DF                ; force upper case
ASPname3:cmp    DSP_ASP_l+2
        bne     notASP
        lda     #1
        sta     DSPisASP
notASP: rts


;;----------------- select a DSP program from dial selection by user
;;---      dynamically builds a message table contianing the names
;;---      of all DSP programs in the image (Unless there is only one pgm)
;;---      It displays the message table and get the users selection
;;---      If only one pgm in image then returns index to it directly
;;---         arguments: none
;;---         returns:    Index to selected program in Acc is succsessful
;;--                      Carry clear if program is selected
;;--                      Carry set if no DSP programs are in SEEPROM

DSP_pgm_select:
        jsr     LCD_Clear
        ldhx    #Select_msg
        jsr     disp_msg                ;display "Band Selection" on 1st line
        ldhx    #SelectCmndTable          ;set up pointer to CmndTable
        sthx    CmndTabStart
;   build msg table dynamically
        jsr     DSPreadDirChkSig          ; read DIR and check sig
        bcc     buildTab                  ; sig OK proceeed
                                          ; sig not OK let user know
        jmp     RunFailAuto1

buildTab1:
        clra                      ; only one program => index is 0
        clc                       ; indicate success
        rts
buildTab:
        lda     {SPI_buffer1+PGMCOUNT} ; file count
        cmp     #1                     ; only one => job done
        beq     buildTab1
        psha                           ; save pgm count
        ldhx    #SPI_buffer2
        sthx    to_addr
        ldhx    #{SPI_buffer1+DIRSTART}
bc1:    pshh                           ; save dir start address
        pshx
        sthx    from_addr
        jsr     StringCopy        ; StringCopy leaves to_addr one beyond string
        dec     3,sp              ; decrement pgm count
        beq     bc2               ; done?
        pulx                      ; advance to next dir entry
        pulh                      ; by getting current dir start
        aix     #DIRSIZE          ; add dir size
        bra     bc1

bc2:    lda     #0
        ldhx    to_addr
        sta     ,x                 ; terminate table with nul
        ais     #3                 ; toss pgm count and h:x from stack
        ldhx    #SPI_buffer2       ;set up pointer to MsgTable we built
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jmp     get_dial_selection

Select_msg:  fcb     'Select DSP pgm',0

;;--- the message table was built dynamically in SPI_buffer2

;;--- the command table has 10 entries (max # of pgms in image)
SelectCmndTable:
        fdb     do_pgm_select
        fdb     do_pgm_select
        fdb     do_pgm_select
        fdb     do_pgm_select
        fdb     do_pgm_select
        fdb     do_pgm_select
        fdb     do_pgm_select
        fdb     do_pgm_select
        fdb     do_pgm_select
        fdb     do_pgm_select
        fdb     do_pgm_select
        fdb     do_pgm_select
;;--- the command processor
do_pgm_select:
        lda     msg_cntr          ; get selected menu item number
        clc                       ; indicate success
        rts


DSPreadDirChkSig:               ; reads DIR into SPI_buffer1
                                ; returns carry clear if sig OK
                                ;         carry set if not oK
        jsr     DSPread_DIR
        ldhx    #0                  ; chk signature in DIR
        lda     #4
        psha
chksigD:
        lda     SPI_buffer1,x
        cmpa    DSP_sig,x
        bne     chksigDfail
        aix     #1
        dec     1,sp
        bne     chksigD
        pula
        clc
        rts
chksigDfail:
        pula
        sec
        rts

DSPread_Dir:
        jsr     SPI_Init
        ldhx    #SPI_buffer1       ; read directory
        sthx    SPI_RamAddr
        lda     DSP_offset         ; start of DSP programs
        sta     SPI_offset         ; to offset for read directory
        lda     DSP_offset+1       ; start of DSP programs
        sta     SPI_offset+1       ; to offset for read directory
        lda     #{DIRSTART+10*DIRSIZE} ; read file header plus 10 max entries
        ldhx    #SPI_Args
        jsr     SPI_Read
        rts

findDSP:
        jsr     SPI_Init
        jsr     SPI_DeviceReady     ; read back last page
        ldhx    #SPI_buffer1
        sthx    SPI_RamAddr
        mov     #$ff,SPI_Offset
        mov     #$80,SPI_Offset+1
        ldhx    #SPI_Args
        lda     #$80
        jsr     SPI_Read
        lda     SPI_buffer1+$7E     ; Pickup offset to start of image
        sta     DSP_offset         ; may be bogus but will be verified shortly
        lda     SPI_buffer1+$7F
        sta     DSP_offset+1
        rts

DSP_dsp908_l: fcb  'DSP908'
DSP_ASP_l    fcb  'ASP'
DSP_GoOk:  fcb   'DSP Pgm started OK',CR,LF,0
DSP_NoGO:  fcb   'DSP pgm failed to start',CR,LF,0
DSP_DL_ok:  fcb  'DSP download checksum OK',CR,LF,0
DSP_DL_fail:  fcb  'DSP download checksum failed',CR,LF,0
DSP_aliveNotOKmsg: fcb 'DSP alive test invalid response',CR,LF,0
DSP_aliveOKmsg:   fcb 'DSP alive test OK',CR,LF,0
DSP_notOKmsg fcb  ' not OK',CR,LF,0
DSP_OKmsg:   fcb  'OK',CR,LF,0
DSPmsg1:  fcb  CR,LF,"DSP load start",CR,LF,0
DSPmsg2:  fcb  "DSP load complete",0
