;******************************************************************************
;                   Routines to Support SEEPROM Binary Data Transfer
;                      Written by Steve Holton N1NB  December 2004
;                          SEEPROM -> PC and PC -> SEEPROM
;******************************************************************************


;-------------------- Receive data from PC needing count -------------------------------
;             On Entry:  2 word argument list in low memory already setup
;                          by caller at label SPI_XferArgs:
;                        1st 16 bit word SEEPROM offset where data to be stored;
;                        2nd 16 bit word byte count IGNORED on entry
;                            will be filled in with count from obtained from PC
;                        Accumulator contains data type (single character)
;             On Exit:   A=0 success
;                        A=1 'ok' not received from PC
;                        2nd word of argument list now contains byte count from PC
;                        H:X destroyed
;             Utilizes:  SPI_buffer1  256 byte buffer in RAM
;                        SPI_XferArgs in Low RAM
;                        SPI_Args in Low RAM

SPI_DataRcvNC:
        psha                        ; save type for now
        jsr     SPI_Init            ; initialize SPI module

        ldhx    #SPImsg_rcv         ; Send 'zZ'for Receive
        jsr     _puts
        pula                        ; recover type
        jsr     _putch              ; send type from A (via stack)
        ldhx    #SPImsg_NC          ; Send '??'to get count
        jsr     _puts

        jsr     getCount            ; get count from PC
        bra     dataRcvCommon

;-------------------- Receive data from PC -----------------------------------
;             On Entry:  2 word argument list in low memory already setup
;                          by caller at label SPI_XferArgs:
;                        1st 16 bit word SEEPROM offset where data to be stored
;                        2nd 16 bit word byte count
;                        Accumulator contains data type (single character)
;             On Exit:   A=0 success
;                        A=1 'ok' not received from PC
;                        H:X destroyed
;             Utilizes:  SPI_buffer1  256 byte buffer in RAM
;                        SPI_XferArgs in Low RAM
;                        SPI_Args in Low RAM

SPI_DataRcv:
        ldhx    #SPImsg_rcv         ; Send 'zZ'for Receive
        jsr     xferInit

        jsr     _getchne
        cmp     #'o'
        bne     dataXfer_err1
        jsr     _getchne
        cmp     #'k'
        bne     dataXfer_err1
dataRcvCommon:
        lda     #'g'
        jsr     _putch
        lda     #'o'
        jsr     _putch
MdataRcvCommon:                 ; entered here for manual download of DSP pgm
        ldhx    #SPI_buffer1    ; set RAM buffer ptr to start
        sthx    SPI_RamAddr
        lda     SPI_CountHi     ; test for byte count all zeros
        ora     SPI_CountLo     ; if so receive one extra time
        beq     rcvXtra1        ; to get full 256 rcvs of 256 bytes
rcvLoop1:
        lda     SPI_CountHi     ; request more than 256 bytes?
        beq     rcvLast         ; branch if no
rcvXtra1:
        sub     #1              ; decrement count by 256
        sta     SPI_CountHi
        lda     #$80
        jsr     RcvWriteAdv
        lda     #$80
        jsr     RcvWriteAdv
        bra     rcvLoop1

rcvLast:                       ; We have less than 256 bytes to go
        lda     SPI_CountLo    ; last chunk - receive any remaining bytes if any
        beq     rcvExitOK      ; there are none - we're done
        cmp     #$80           ; Do we have one page or more left?
        beq     rcvNoFill      ; exactly 128 left no padding needed
        blo     rcvpad         ; less than 128 so pad out buffer
        lda     #$80           ; more than 128 get the next 128
        bra     rcvNoFill
rcvPad:                        ; less than 128 bytes remaining
        ldhx    #SPI_Buffer1
        lda     ManualDl        ; manual DL?                        
        beq     rcvPadNorm      ; no carryon
        aix     #7              ; skip over 7 we read
rcvPadNorm:        
        lda     #$00           ; last block partial page fill buffer with 00's first
rcvFill:
        sta     ,x
        aix     #1
        cphx    #SPI_Buffer1+128  ; we've filled buffer do receive
        bne     rcvFill
        lda     SPI_CountLo
rcvNoFill:
        jsr     RcvWriteAdv
        lda     SPI_CountLo       ; check size of any remaining chunk ?
        beq     rcvExitOK         ; there is no more we're done
        cmp     #$80              ; is last chunk >= 128
        bls     rcvExitOK         ; last chunk exactly 128 or less we've done it
        sub     #$80              ; More than 128 reduce count by 128
        sta     SPI_CountLo       ; save remaining count
        bra     rcvPad            ; go get last bit
rcvExitOK:
        ldhx    SPI_count2
        sthx    SPI_count         ; restore actual byte count to argument list
        lda     #'o'
        jsr     _putch
        lda     #'k'
        jsr     _putch
        clra
        rts

;-----------------------------  common to send and receive ------------------
;       placed here so can be reached directly by send & receive via conditional branches
dataXfer_err2:
        lda     #2
        bra     XferExiterr
dataXfer_err1:
        lda     #1
XferExiterr:
        jsr        _crlf
        ldhx       #SPIxferErrMsg1
        jsr        prt_msg_byte
        jsr        _crlf
        rts

;-------------------- Send data to PC  ----------------------------------------
;             On Entry:  2 word argument list in low memory already setup
;                          by caller at label SPI_XferArgs:
;                        1st 16 bit word SEEPROM offset where data to obtained
;                        2nd 16 bit word byte count
;                        Accumulator contains data type (single character)
;             On Exit:   A=0 success
;                        A=1 'go' not received from PC
;                        A=2 'ok' not received from PC
;                        H:X destroyed
;             Utilizes:  SPI_buffer1  256 byte buffer in RAM
;                        SPI_XferArgs in Low RAM
;                        SPI_Args in Low RAM

SPI_DataSend:
        ldhx    #SPImsg_send    ; Send 'zz' for send
        jsr     xferInit        ; do prep. and send request

        jsr     _getchne        ; wait for 'go' from PC
        cmp     #'g'
        bne     dataXfer_err1
        jsr     _getchne
        cmp     #'o'
        bne     dataXfer_err1
dataSendCommon:
        lda     SPI_CountHi     ; test for all zeros
        ora     SPI_CountLo     ; if so send one extra time
        beq     sendXtra1       ; to get full 256 sends of 256 bytes
sendLoop1:
        lda     SPI_CountHi     ; request more than 256 bytes?
        beq     sendLast        ; branch if no
sendXtra1:
        sub     #1              ; decrement count by 256
        sta     SPI_CountHi
        ldhx    #SPI_Args
        lda     #$00            ; read 256 bytes
        jsr     SPI_Read
        lda     #$00
        jsr     sendBlock
        ldhx    SPI_Offset      ; increment SEEPROM offset by 256
        aix     #$60
        aix     #$60
        aix     #$40
        sthx    SPI_Offset
        bra     sendLoop1
sendLast:
        lda     SPI_CountLo    ; last chunk send any remaining bytes if any
        beq     SendExitOK     ; there are none - we're done
        ldhx    #SPI_Args
        jsr     SPI_Read
        lda     SPI_CountLo
        jsr     sendBlock
SendExitOK:
        ldhx    SPI_count2
        sthx    SPI_count         ; restore actual byte count to argument list
        jsr     _getchne
        cmp     #'o'
        bne     dataXfer_err2
        jsr     _getchne
        cmp     #'k'
        bne     dataXfer_err2
        clra
        rts

;-------------------- Send data to PC needing count -------------------------------
;             On Entry:  2 word argument list in low memory already setup
;                          by caller at label SPI_XferArgs:
;                        1st 16 bit word SEEPROM offset where data to be obtained
;                        2nd 16 bit word byte count IGNORED on entry
;                            will be filled in with count from obtained from PC
;                        Accumulator contains data type (single character)
;             On Exit:   A=0 success
;                        A=2 'ok' not received from PC
;                        2nd word of argument list now contains byte count from PC
;                        H:X destroyed
;             Utilizes:  SPI_buffer1  256 byte buffer in RAM
;                        SPI_XferArgs in Low RAM
;                        SPI_Args in Low RAM

SPI_DataSendNC:
        psha                        ; save type for now
        jsr     SPI_Init            ; initialize SPI module

        ldhx    #SPImsg_send        ; Send 'zz'for Receive
        jsr     _puts
        pula                        ; recover type
        jsr     _putch              ; send type from A (via stack)
        ldhx    #SPImsg_NC          ; Send '??'to get count
        jsr     _puts

        jsr     getCount            ; get count from PC
        bra     dataSendCommon

;-----------------------SPI Data Transfer Subroutines -----------------------

;---------------------receive block of data from PC --------------------------
rcvBlock:                   ; receive bytes # of bytes A (00=256, 1-255=1-255)
        psha                    ; put count on stack                    
        ldx     ManualDl        ; manual DL?                        
        beq     rcvBlockNorm    ; no carryon
        sub     #7              ; yes reduce count by 7 we read already
        sta     1,sp            ; update count on stack
        ldhx    #SPI_buffer1+7  ; start after 7 we read already
        clra    
        sta     ManualDL       ; do only first time through
        bra     rcvBlockLoop        
rcvBlockNorm:
        ldhx    #SPI_Buffer1    ; data to go in SPI_Buffer1 (& SPI_Buffer2)
rcvBlockLoop:
        jsr     _getchne
        sta     ,x              ; get next byte
        aix     #1
        lda     1,sp            ; get count
        deca                    ; decrement count 1
        beq     rcvBlockDone    ; branch if done
        sta     1,sp
        bra     rcvBlockLoop
rcvBlockDone:
        pula                    ; restore stack & return 0
        rts

;---------------------send block of data to PC --------------------------
sendBlock:                  ; send bytes # of bytes A (00=256, 1-255=1-255)
        psha                    ;  put count on stack
        jsr     wait10ms        ;  TIME xmit delay
        ldhx    #SPI_Buffer1    ; data in SPI_Buffer1 (& SPI_Buffer2)
sendBlockLoop:
        lda     ,x              ; get next byte
        jsr     _putch          ; send it
        aix     #1
        lda     1,sp            ; get count
        deca                    ; decrement count 1
        beq     sendBlockDone   ; branch if done
        sta     1,sp
        bra     sendBlockLoop
sendBlockDone:
        pula                    ; restore stack & return 0
        rts
;--------------------receive nn (in ACC) bytes, write one page, advance offset
RcvWriteAdv:
        jsr     rcvBlock
        jsr     SPI_DeviceReady
        jsr     SPI_WriteEnable
        ldhx    #SPI_Args
        jsr     SPI_Writepage
        ldhx    SPI_Offset      ; increment SEEPROM offset by 128
        aix     #$20
        aix     #$60
        sthx    SPI_Offset
        rts

getCount:
        jsr     _getchne        ; get count from PC
        sta     SPI_Count
        sta     SPI_Ptr
        jsr     _getchne
        sta     SPI_Count+1
        sta     SPI_Ptr+1
        rts

xferInit:         ; save count, SPI Init, send request msg in H:X, send count
        psha
        jsr     SPI_Init            ; initalize SPI module
        lda     SPI_count           ; save byte count
        sta     SPI_Ptr
        lda     SPI_count+1
        sta     SPI_Ptr+1
        jsr     _puts               ; send request msg
        pula                        ; recover type
        jsr     _putch              ; send type from A (via stack)
        lda     SPI_XferArgs+2      ; send typr from SPI_Args
        jsr     _putch
        lda     SPI_XferArgs+3
        jsr     _putch
        rts

prt_msg_byte:              ; Prints MSG in H:X followed by the byte in A and crlf
        psha
        jsr     _puts
        pula
        jsr     _byte_sp
        jsr     _crlf
        rts

SPIxferErrMsg1: fcb   'transfer error aborting',0
SPImsg_send: fcb 'zz',0
SPImsg_rcv:  fcb 'zZ',0
SPImsg_NC:   fcb '??',0


