$page
;*****************************************************************************
;*****************************************************************************
;*                                                                           *
;*                         DRIVERS_M MODULE                                  *
;*                                                                           *
;*                for Micro908 with 8-bit display I/F                        *
;*             Uses port D for data, and port A for control                  *
;*                                                                           *
;*    January 2005 Steve Holton N1NB                                         *
;*      added DSP908 & ASP PB "toggling" hooks                               *
;*      Fixed bug in Cmnd2LCD                                                *
;*      Modified Get_EEPROM for Page2 support                                *
;*      Added Erase_EEPROM_P2,Put_EEPROM_P2
;*****************************************************************************
;*****************************************************************************
;
; This module contains drivers for the following components:
;      DDS
;         Set_DDS
;      Shaft Encoders (Special and Grey)
;         Read_Encoder
;      ADC
;         Read_ADC
;      Piezo
;         MakeSWRtone
;      LCD
;         Cmnd2LCD
;         Data2LCD
;         BusyCheck
;         LCD_Init
;         LCD_Clear
;         ClearLCDLine1
;         ClearLCDLine2
;         BlankLCD
;         UnBlankLCD
;         ShowCursor
;         HideCursor
;         Disp_Msg
;      EEPROM
;         Write_EEPROM_Byte
;         Erase_EEPROM_All
;         Erase_EEPROM_Byte
;         Init_EEPROM
;         EEPROM_On
;         EEPROM_Off
;         EEPROM_Security_On
;         EEPROM_Security_Off
;         EEPROM_Disable
;         EEPROM_Enable
;	  Get_EEPROM
;	  Put_EEPROM
;
;*****************************************************************************

$IF  AA908
;*****************************************************************************
; SET_DDS - This routine sends the 5-byte DDS control word (DDS_w4 through
;           DDS_w0, LSB to MSB in each) to the DDS chip by serial data transfer.

set_dds:
        ldhx    #DDS_w4           ; point to the start of the DDS programming words
        mov     #$05,byte_count   ; we'll process 5 bytes

nextbyt: mov     X+,byte2send      ; get the DDS word (and increment pointer H:X)
        mov     #$08,bit_count    ; we'll process 8 bits

nextbit: ror     byte2send         ; test next LSB for 0 or 1
        bcc     _send0             ; send a 0 if carry clear

_send1:  bset    DDS_data,portf    ; else send a one
        bset    DDS_clk,portf     ; Toggle write clock
        bclr    DDS_clk,portf
        bra     _check

_send0:  bclr    DDS_data,portf    ; Send zero
        bset    DDS_clk,portf     ; Toggle write clock
        bclr    DDS_clk,portf

_check:  dbnz    bit_count,nextbit  ; get next DDS programming bit if not done
        dbnz    byte_count,nextbyt ; get next DDS programming byte if not done

        bset    DDS_load,portf    ; else done - send DDS load signal
        bclr    DDS_load,portf
        rts
$ENDIF

$page
;*****************************************************************************
;**************     SHAFT ENCODER DRIVER     *********************************
;*****************************************************************************

;*****************************************************************************
; Read Encoder - Wait here for shaft to turn.  Exit with
;                             Z=0 if CCW
;                             Z=1 if CW
;                             CY=1 if pushbutton depressed
;                             CY=0 if not
;                Enter with ACC set for desired rotary sensitivity (that many
;                encoder transitions need to happen before the routine exits with
;                CW or CCW indication.)
;*****************************************************************************
;             +     +
;       ----  +   --+-      ----
;  A        | +  |  + |    |    |
;           | +  |  + |    |    |
;            -+--   +  ----
;             +     +
;          ---+     +---      ----
;  B          +|   |+   |    |
;             +|   |+   |    |
;             + --- +    ----
;             +     +
;*****************************************************************************
             ; Await negative transition on either A or B shaft encoder line

Read_Encoder:
        sta     sens_sav              ;save entered encoder Sensitivity value
        sta     sens_cw               ;set CW sensitivity control
        sta     sens_ccw              ;set CCW sensitivity control

re0:    nop
;       jsr     CheckPwrPB            ;check Power PB for power down request

$IF     AF908
        psha                          ; SMH
        lda     DSPisdsp908           ; SMH if dsp908 is running
        beq     re0a                  ; SMH
        jsr     Checkdsp908_PB        ; SMH check for toggling via PBs
re0a:   lda     DSPisASP              ; SMH if ASP is running
        beq     re0b                  ; SMH
        jsr     CheckASP_PB           ; SMH check for toggling via PBs
re0b:   pula
$ENDIF

        brclr   ENC_PB,portF,re5      ;if pushbutton depressed, exit with CY=1
        brset   ENC_A,portF,re1       ;A still high, go check B
        tst     A_flag                ;A is low ... previously low too?
        beq     re2                   ;yes, previously low so not neg-edge. Check B next.
        clr     A_flag                ;neg edge A found!  Set flag to indicate it's now low
        bra     re4                   ;and go process rotation

re1:    mov     #$FF,A_flag           ;set A flag to indicate that it's (still) high

re2:    brclr   ENC_B,portF,re3       ;B is low ... continue checking it
        mov     #$FF,B_flag           ;B is high, so set flag indicating this
        bra     re0                   ;and do all again
re3:    tst     B_flag                ;B previously low?
        beq     re0                   ;yes, nothing to process, so do all over again
        clr     B_flag                ;neg edge B found! Set flag indicate it's now low
                                      ;and continue onward to process direction

             ;Negative edge found on A or B.  Determine which direction shaft was turned.
             ;         Direction = CW if B toggled
             ;         Direction = CCW if A toggled
re4:    mov     enc_new,enc_old
        lda     portF
        and     #$30
        sta     enc_new
        eor     enc_old
        and     #$10
        beq     CCW                     ;z=1 ==> A toggled = CCW rotation = down

             ; CW rotation sensed.
        lda     sens_sav
        sta     sens_ccw                ;Reset CCW Sensitivity cntr
        dbnz    sens_cw,re0             ;wait for SENS_CW transitions to occur before exiting
        lda     #1                      ;set Z=0 for CW rotation
        clc                             ;CY=0 for pushbutton not depressed
        rts

             ;CCW rotation sensed
CCW:    lda     sens_sav
        sta     sens_cw                 ;Reset CW Sensitivity cntr
        dbnz    sens_ccw,re0            ;wait for SENS_CCW transitions to occur before exiting
        lda     #0                      ;Set Z=1 for CCW rotation
        clc                             ;CY=0 for pushbutton not depressed
        rts

             ;exit with CY=1 if pushbutton pressed
re5:    sec                             ;set CY to indicate pushbutton pressed
        rts

;**************************************************************************
;*******************      ADC Driver      *********************************
;**************************************************************************

; Read_ADC -- Desired port to be read is in A (0=Vr, 1=Vz, 2=Vf, 3=Batt)

Read_ADC:
        sta     ADSCR                 ;clear COCO, AIEN, ADCO, specify port #, and start a conversion
        brclr   7,ADSCR,*             ;wait for Conversion Complete (COCO)
        lda     ADR                   ;read the A/D register
        rts

;            ; No Bias version
;Read_ADC:
;        psha
;        jsr     wait1ms
;        pula
;        sta     ADSCR                 ;clear COCO, AIEN, ADCO, specify port #, and start a conversion
;        brclr   7,ADSCR,*             ;wait for Conversion Complete (COCO)
;        lda     ADR                   ;read the A/D register
;        cmp     #$10                  ;only keep A/D reading if it's > $10  (avoids ADC dead band)
;        bcc     r_exit
;        clra
;r_exit: psha
;        jsr wait1ms
;        pula
;        rts

$IF AA908
;**************************************************************************
;*******************        Speaker Control      **************************
;**************************************************************************
;------------------------------------------
;MakeSWRtone - compute the tone frequency

MakeSWRtone:
        lda     iCalc_flag           ;is "interrupt Calc flag" on? (Set in Process_Dial during dial static times)
        bne     mt1                  ;yes, we're operating in the interrupt routine and doing the offset Vr readings
                                     ; so we don't want to change the primary tone value
        lda     #CHAN_Vr             ;else, we're on the displayed freq. Make the TONE variable a function of Vr
        jsr     read_ADC
        sta     Tone_freq
        lda     #$FF
        sub     Tone_freq
        sta     Tone_freq
mt1:    rts
$ENDIF

;**************************************************************************
;*******************      LCD Backlight Control   *************************
;**************************************************************************

;------------------------------------------------------------
;TurnLiteOn - Turn on the LCD backlite

TurnLiteOn:
        bset    Backlite,portC
        rts

;------------------------------------------------------------
;TurnLiteOff - Turn off the LCD backlite

TurnLiteOff:
        bclr    Backlite,portC
        rts

;------------------------------------------------------------
ResetAutoTimers:
        lda     LiteOffMins             ;re-init the LiteOff minutes counter from the constant
        sta     LiteOffMinCtr
        lda     LiteOnFlag
        beq     rat1
        jsr     TurnLiteOn              ;yes, turn it back on
        bra     rat2
rat1:   jsr     TurnLiteOff
rat2:   rts

;**************************************************************************
;*******************      LCD Driver      *********************************
;**************************************************************************

;------------------------------------------------------------
;Display character in A to the LCD.  LCD_rw and LCD_rs must be
;set up prior to entry.

Cmnd2LCD:
        sta     LCD_char        ;temp save char to be written
        bclr    LCD_rs,porta    ;rs is low for cmnd
        jsr     busy_check      ;LCD ready for new data?
        bclr    LCD_e,porta     ; SMH needed?? replaced clra which resets DSP
        mov     #$FF,ddrd       ;setup for all outputs
        bclr    LCD_rw,porta    ;change LCD back to Write mode
        bclr    LCD_rs,porta    ;ensure that rs is low for cmnd
        bra     write_LCD       ;continue
Data2LCD:
        sta     LCD_char        ;temp save char to be written
        bset    LCD_rs,porta    ;rs is hi for data
        jsr     busy_check      ;LCD ready for new data?
        bclr    LCD_e,porta     ; SMH needed?? replaced clr porta which resets DSP
        mov     #$FF,ddrd       ;setup for all outputs
        bclr    LCD_rw,porta    ;change LCD back to Write mode
        bset    LCD_rs,porta    ;ensure that rs is hi for data
Write_LCD:
        lda     LCD_char        ;get the character
        sta     portd
        bset    LCD_e,porta     ;strobe enable
        jsr     wait1us
        bclr    LCD_e,porta
        rts

;-----------------------------------------------------
;Busy_check Subroutine
;
;LCD read/write operations are slooooow, this subroutine
;polls the LCD busy flag to determine if previous operations are completed.

Busy_Check:
        clra
        mov     #$0F,ddrd       ;tristate the data pins
        bclr    LCD_rs,porta    ;set up to read busy flag
        bset    LCD_rw,porta    ;set up for Read
        jsr     wait1us
        mov     #$FF,LCD_timer  ;init timeout counter
busy:   bset    LCD_e,porta     ;raise the Enable bit
        jsr     wait1us
        mov     portd,LCD_read  ;read the byte
        bclr    LCD_e,porta
        dbnz    LCD_timer,busy2 ;timer not zero
        bra     not_busy        ;timer=0 so exit anyway
busy2:  brset   7,LCD_read,busy ;check for busy flag - wait if set

not_busy:
        rts

;-----------------------------------------------------------
; LCD_INIT
; Power on initialization of LCD.
; The LCD controller chip must be equivalent to an Hitachi 44780.

LCD_Init:
        bclr    LCD_rw,porta          ;r/w is low for write
        bclr    LCD_rs,porta          ;rs is low for cmnd
        jsr     wait100ms             ;wait for pwr supply to stabilize

        mov     #$30,portd            ;func set
        bset    LCD_e,porta
        jsr     Wait1us
        bclr    LCD_e,porta
        jsr     Wait10ms

        mov     #$30,portd            ;func set
        bset    LCD_e,porta
        jsr     Wait1us
        bclr    LCD_e,porta
        jsr     Wait10ms

        mov     #$30,portd            ;func set
        bset    LCD_e,porta
        jsr     Wait1us
        bclr    LCD_e,porta
        jsr     Wait10ms

        lda     #$3C                  ;8-bit mode, 2 lines, 5x8 matrix chars
        jsr     cmnd2LCD

        lda     #$08                  ;disp off, curs & blank off
        jsr     cmnd2LCD

        lda     #$01                  ;clear & reset cursor
        jsr     cmnd2LCD

        lda     #$06                  ;cursor moves right, no shift
        jsr     cmnd2LCD

        lda     #$0C                  ;display on, cursor & blink off
        jsr     cmnd2LCD

        rts

;-----------------------------------------------------------
; LCD_clear -- clears the LCD display

LCD_Clear:
        lda     #$01
        jsr     cmnd2LCD
        rts

;-----------------------------------------------------------
; ClearLCDLine1 -- point to 1st char on Line 1, clear line, leave cursor at position 1

ClearLCDLine1:
        lda     #$80		      ;point to 1st char line 1
	jsr	Cmnd2LCD
	jsr	wait1ms
	ldhx    #_blank_line
	jsr	disp_msg
	lda     #$80		      ;leave cursor at home on line 1
	jsr	Cmnd2LCD
	rts

_blank_line     fcb     "                    ",0

;-----------------------------------------------------------
; ClearLCDLine2 -- point to 1st char on Line 2, clear line, leave cursor at position 1

ClearLCDLine2:
        lda     #$C0		      ;point to 1st char line 2
	jsr	Cmnd2LCD
	jsr	wait1ms
	ldhx    #_blank_line
	jsr	disp_msg
	lda     #$C0		      ;point to 1st char line 2
	jsr	Cmnd2LCD
	rts

;-----------------------------------------------------------
; BlankLCD

BlankLCD:
        lda     #8		      ;Blank the LCD
	jsr     Cmnd2LCD
	rts

;-----------------------------------------------------------
; UnBlankLCD

UnBlankLCD:
        lda     #12		      ;UnBlank the LCD
	jsr     Cmnd2LCD
	rts

;-----------------------------------------------------------
; ShowCursor -- produce a blinking cursor

ShowCursor:
        lda     #15		      ;Show Cursor (blinking)
        jsr     Cmnd2LCD
	jsr     wait1ms
	rts

;-----------------------------------------------------------
; HideCursor

HideCursor:
        lda     #12		       ;Hide Cursor
        lda     Cmnd2LCD
	jsr     wait1ms
	rts

;-----------------------------------------------------------
; DISP_MSG -- displays the message pointed to by H:X

Disp_Msg:
        lda     ,x
        beq     disp_msg_end           ;zero ends the string
        jsr     data2LCD
        aix     #1
        bra     disp_msg

disp_msg_end:
        rts

$page
;**************************************************************************
;*******************      EEPROM Driver      ******************************
;**************************************************************************
;
;--------------------------------------------------------------------------
; Write_EEPROM_Byte -- The target EEPROM address H:X (in the range 800-9FF)
;                      is first erased, then written with the byte supplied
;                      in reg A.

Write_EEPROM_Byte:
        psha                            ;save the data to be written
        jsr     Erase_EEPROM_Byte       ;erase the target location
        lda     #(EERAS1|EERAS0)
        coma                            ;clear EERAS1 & EERAS0 in EECR
        and     EECR
        sta     EECR
        ora     #EELAT                  ;set EELAT in EECR
        sta     EECR
        pula                            ;and retrieve the data
        sta     ,x                      ;Write data to the desired EEPROM address
        lda     #EEPGM                  ;Set the EEPGM bit in the EECR
        ora     EECR
        sta     EECR
        jsr     Wait100ms                ;Wait 10ms to program the byte.
        lda     #EEPGM                  ;clear EEPGM
        coma
        and     EECR
        sta     EECR
        jsr     Wait100us               ;Wait 100us for pgm voltage to fall.
        lda     #EELAT
        coma                            ;clear EELAT
        and     EECR
        sta     EECR
        rts

;--------------------------------------------------------------------------
; Erase_EEPROM_All -- Bulk erase all EEPROM locations (800-9FF)

Erase_EEPROM_all:
        ldhx    #$800                   ;any location in EEPROM area will do bulk erase
        lda     EECR
        ora     #(EERAS1|EERAS0)
        sta     EECR
        bra     ee_1

;--------------------------------------------------------------------------
; Erase_EEPROM_P0 -- Bulk erase the 128 Page 0 EEPROM locations (800-87F) = Config Data

Erase_EEPROM_P0:
        ldhx    #$800                   ;any location in Page 0 will do bulk erase
        lda     EECR
        and     #$F7                    ;drop EERAS0
        ora     #EERAS1                 ;raise EERAS1
        sta     EECR
        bra     ee_1

;--------------------------------------------------------------------------
; Erase_EEPROM_All -- Bulk erase the 128 Page 1 EEPROM locations (880-8FF) = Calibration Data

Erase_EEPROM_P1:
        ldhx    #$880                   ;any location in Page 1 will do bulk erase
        lda     EECR
        and     #$FE                    ;drop EERAS0
        ora     #EERAS1                 ;raise EERAS1
        sta     EECR
        bra     ee_1

$IF  AF908
;------------------ SMH ---ADDED-Not Currently USED------- SMH -------------------------------------
; Erase_EEPROM_P2 -- Bulk erase the 128 Page 2 EEPROM locations (900-87F) = DSP Data

;Erase_EEPROM_P2:
;        ldhx    #$900                   ;any location in Page 0 will do bulk erase
;        lda     EECR
;        and     #$F7                    ;drop EERAS0
;        ora     #EERAS1                 ;raise EERAS1
;        sta     EECR
;        bra     ee_1
$ENDIF

;--------------------------------------------------------------------------
; Erase_EEPROM_Byte -- Erase target EEPROM address H:X (in the range 800-9FF)

Erase_EEPROM_byte:
        lda     EECR
        and     #$EF                    ;clear EERAS1 (b4)
        ora     #$08                    ;set EERAS0 (b3)
        sta     EECR
ee_1:   ora     #EELAT                  ;set EELAT in EECR
        sta     EECR
        sta     ,x                      ;write any data to the target location erases
        lda     #EEPGM                  ;Set the EEPGM bit in the EECR
        ora     EECR
        sta     EECR
        jsr     Wait100ms                ;Wait 10ms for byte erase
        lda     #EEPGM
        coma                            ;clear EEPGM
        and     EECR
        sta     EECR
        jsr     Wait100us               ;Wait 100us for pgm voltage to fall.
        lda     #EELAT
        coma                            ;clear EELAT
        and     EECR
        sta     EECR
        rts

;--------------------------------------------------------------------------
; Init_EEPROM -- Initialize the EEPROM configuration registers
;                This is a nonvolatile register setting

Init_EEPROM:
        jsr     EEPROM_On               ;Turn on array for operation

        lda     #$10                    ;lower 4 bits are positions for EEPROM bank enables
        ldhx    #EENVR
        jsr     Write_EEPROM_Byte       ;enable the EEPROM
        lda    ,x                       ;puts new settings into effect

        lda     #$80                    ;set upper bit EEDIVSECD to allow the values to be modified later
        ldhx    #EEDIVHNVR
        jsr     Write_EEPROM_Byte       ;write the high divider byte
        lda     #172                    ;divider value is 170 (based on CGMXCLK clock of 4.9152 MHz)
        ldhx    #EEDIVLNVR
        jsr     Write_EEPROM_Byte       ;write the low divider byte

        rts

;--------------------------------------------------------------------------
; EEPROM_On -- Turns on the EEPROM array for operation (default = ON)

EEPROM_On:
        lda     #EEOFF
        coma
        and     EECR                    ;clear EEOF in EECR
        sta     EECR
        rts

;--------------------------------------------------------------------------
; EEPROM_Off -- Turns off the EEPROM array.  Provides for lower power
;               operation (default = ON)

EEPROM_Off:
        lda     EECR
        ora     #EEOFF                  ;set EEOFF on EECR
        sta     EECR
        rts

;--------------------------------------------------------------------------
; EEPROM_Security_On -- Turns on the EEPROM array security bit
;                       This is a nonvolatile setting. (Initially = disabled)
;                       Takes effect after reset.

EEPROM_Sec_On:
        mov    EEPRTCT,temp             ;set up temp to hold inverse of EEPRTCT bit
        com    temp                     ; to enable clearing of bit in EENVR below
        ldhx   #EENVR
        lda    ,x
        and    temp                     ;0=secure
        jsr    Write_EEPROM_Byte
        rts

;--------------------------------------------------------------------------
; EEPROM_Security_Off -- Turns off the EEPROM array security bit
;                        This is a nonvolatile setting. (Initially = disabled)
;                        Changes take effect after reset.

EEPROM_Sec_Off:
        ldhx   #EENVR
        lda    ,x
        ora    #EEPRTCT                 ;1=security disabled
        jsr    Write_EEPROM_Byte
        lda    ,x                       ;puts new settings into effect
        rts

;--------------------------------------------------------------------------
; EEPROM_Disable -- Protects the selected EEPROM block (0,1,2,3)
;                   from being written or erased.  Block indicated
;                   in corresponding bits of reg A.
;                   All four blocks are initially unprotected. (3:0 =0)
;                   This is a nonvolile setting. (Initially = disabled)
;                   Changes take effect after a reset.

EEPROM_Disable:
        and    #$0F                     ;isolate the block bits (3:0)
        sta    temp                     ;temp save selected block(s)
        ldhx   #EENVR
        lda    ,x
        ora    temp                     ;disable (protect) the selected bits/banks
        jsr    Write_EEPROM_Byte
        lda    ,x                       ;puts new seetings into effect
        rts

;--------------------------------------------------------------------------
; EEPROM_Enable -- Enables the selected EEPROM block (0,1,2,3)
;                  to be being written or erased.  Block indicated
;                  in corresponding bits of reg A.
;                  All four blocks are initially unprotected. (3:0 =0)
;                  This is a nonvolatile setting. (Initially = disabled)
;                  Changes take effect after a reset.

EEPROM_Enable:
        and    #$0F                     ;isolate the block bits (3:0)
        coma                            ;invert (0 means enable)
        sta    temp                     ;temp save selected block(s)
        ldhx   #EENVR
        lda    ,x
        and    temp                     ;enable (unprotect) the selected bits/banks
        jsr    Write_EEPROM_Byte
        lda    ,x                       ;puts new seetings into effect
        rts

;-----------------------------------------------------------------------------
; GET_EEPROM -- Load volatile RAM memory from user's custom settings stored in EEPROM.
;                  If EEPROM is initially blank (i.e., $0800=$FF), then load default
;                  values into EEPROM

Get_EEPROM:
        lda     #$FF
        cmp     $0800                   ;EEPROM erased? (ie, never been used?)
        bne     getE_1                  ;no, load user settings
        jsr     Put_EEPROM_P0           ;first-time save of user settings
        rts                             ;and exit

             ;Load the Page 0 "Config data"
getE_1: ldhx    #$800                   ;start of EEPROM Page 0
        sthx    from_addr
        ldhx    #NV_STRT                ;point to start of bytes to be read 01/21/05 (mlh)
        sthx    to_addr
        mov     #NV_SIZ,EEbyte_count
        jsr     getE_2

$IF AA908
             ;Next load the Page 1 "Calibration data"
        ldhx    #$880                   ;start of EEPROM Page 1
        sthx    from_addr
        ldhx    #Vf_offset              ;destination address
        sthx    to_addr
        mov     #10,EEbyte_count
        jsr     getE_2
$ENDIF
$IF AF908
             ; SMH Load page 2 DSP data
        ldhx    #$8C0                   ;start of EEPROM Page 2
        sthx    from_addr
        ldhx    #Start_P2_data          ;destination address
        sthx    to_addr
        mov     #{End_P2_data-Start_P2_data},EEbyte_count      ;SMH made symbolic
        jsr     getE_2
$ENDIF
        rts
             ;--------------------
             ;Get data from EEPROM
             ;--------------------
getE_2: ldhx    from_addr
        lda     ,x                      ;get char from input stream
        aix     #1
        sthx    from_addr
        ldhx    to_addr
        sta     ,x
        aix     #1
        sthx    to_addr
        dbnz    EEbyte_count,getE_2
        rts

;-----------------------------------------------------------------------------
; PUT_EEPROM_P0 -- Store volatile RAM memory from user's custom settings to EEPROM Page 0

Put_EEPROM_P0:
        jsr     Erase_EEPROM_P0         ;clear all Page 0 locations

Put_EEPROM_P0a:
        ldhx    #$800                   ;start of EEPROM Page 0
        sthx    to_addr

             ;Write "Page 0 data"
        ldhx    #NV_STRT                ;point to start of bytes to be written 01/21/05 (mlh)
        sthx    from_addr
        mov     #NV_SIZ,EEbyte_count
        jsr     ec2
        rts

             ;----------------
             ;Transfer EEbyte_count bytes from "from_addr" to the "to_addr"
             ;----------------
ec2:    ldhx    from_addr
        lda     ,x                      ;get char from input stream
        aix     #1
        sthx    from_addr
        ldhx    to_addr
        jsr     Write_EEPROM_byte       ;write reg A to loc (hx) in EEPROM area
        aix     #1
        sthx    to_addr
        dbnz    EEbyte_count,ec2
        rts

;----------------------------------------------------------------------------------
; PUT_EEPROM_P1 -- Store calibration data from RAM to EEPROM Page 1

Put_EEPROM_P1:
        ldhx    #$880                   ;start of EEPROM Page 1
        sthx    to_addr
        ldhx    #Vf_offset              ;point to start of bytes to be written
        sthx    from_addr
        mov     #10,EEbyte_count
        jsr     ec2
        rts

$IF AF908
;----------------- SMH ---ADDED-------- SMH ---------------------------------------
; PUT_EEPROM_P2 -- Store DSP data from RAM to EEPROM Page 2

Put_EEPROM_P2:
        ldhx    #$8c0                   ;start of EEPROM Page 2
        sthx    to_addr
        ldhx    #Start_P2_data          ;point to start of bytes to be written
        sthx    from_addr
        mov     #{End_P2_data-Start_P2_data},EEbyte_count    ;SMH made symbolic
        jsr     ec2
        rts
$ENDIF
