;*************************************************************
;*****************     TIMER MODULE    ***********************
;**********  includes Delay routines, dummy ISR **************
;*************************************************************

;-------------------------------------------------------------
; Init_Timer_Module - Turns on timer A channel 1 for an Output
;              Compare in 1ms. Creates an interrupt
;              vector to the Timer ISR (T_ISR_A1), where Timeout1,
;              Timeout2 and Timeout3 are decremeted. If a timer
;              reaches 0, then a user-specified operation is
;              done.  Thus, the user can either set a timer in his
;              code and sit there waiting on it to become 0, or
;              he may place the timeout action routine right here
;              in the Timer Handlers.  We use the latter technique
;              for the "heartbeat" LED blink. We set Timeout1 to
;              255 in the Timer1 Handler below, and when 255 ticks
;              of the 1ms timer interrupt occur, we toggle the state
;              of the heartbeat LED and reset Timeout1 to occur again
;              in another 255 ticks.
;---------------------------------------------------------------

Init_Timer_Module:
       mov     #$36,tasc           ; Timer A cleared + stopped. Clicks every 64 bus cycles
       mov     #0,tach1H           ; Set Output Compare to happen in 136 clicks (1 ms)
       mov     #136,tach1L         ;   after we start the timer.
       mov     #0,tach0H           ; Set Output Compare to happen in 65 clicks (0.5 ms)
       mov     #65,tach0L          ;   after we start the timer.
       mov     #$54,tasc1          ; Set Timer A Chan 1 for Output Compare operation
;       mov     #$54,tasc0          ; Set Timer A Chan 0 for Output Compare operation.
       mov     #$06,tasc           ; Start the timer
       rts

;*******************************************************************
; T_ISR_A0 - Timer Interrupt Service Routine for Timer A Chan 2
;               272 clicks = 2.0ms  period =    250 Hz tone
;               136 clicks = 1.0ms  period =    500 Hz tone
;                65 clicks = 0.5ms  period =  1,000 Hz tone
;                13 clicks = 0.1ms  period =  5,000 Hz tone
;                07 clicks = 0.05ms period = 10,000 Hz tone
;*******************************************************************

T_ISR_A0:
       pshh
       pshx
       psha

       bclr    7,tasc0               ;clear OC flag
       ldhx    tach0h
       txa
       add     Tone_freq
       tax
       bcc     ta01
       inch
ta01:  sthx    tach0h                ;setup next interrupt in (h:x) clicks

       pula
       pulx
       pulh
       rti

;*******************************************************************
; T_ISR_A1 - 1ms Timer Interrupt Service Routine for Timer A Chan 1
;*******************************************************************

T_ISR_A1:
       pshh
       pshx
       psha

       bclr    7,tasc1              ;clear OC flag
       ldhx    tach1h
       aix     #127                 ;setup next interrupt in (127+9) clicks (1ms)
       aix     #9
       sthx    tach1h

Check_t1:
       tst     timeout1             ;Timeout 1 currently active?
       beq     check_t2             ;no, go check timer 2
       dec     timeout1             ;yes
       bne     check_t2             ;finished counting down?
       jsr     t1_handler           ;yes - Execute Timeout 1 handler

Check_t2:
       ldhx    timeout2             ;Timeout 2 currently active?
       beq     check_t3             ;no, go check timer 3
       aix     #-1                  ;yes, decrement it
       sthx    timeout2

Check_t3:
       ldhx    PwrDwnRegs
       aix     #-1                  ;decrement 16-bit register
       sthx    PwrDwnRegs           ; (60,000 tics @ 1ms/tick = 60 seconds = one minute)
       bne     done_A1
       jsr     t3_handler           ;yes - Execute Timeout handler

done_A1:
       pula
       pulx
       pulh
       rti

;---------------------------------------------------------------
; Timer A Chan 1 (TACH1) Timeout Handlers
;                    All the user has to do is set one of the
;                    timeout variables to desired delay n (1-255)
;                    and the timeout handler will be executed
;                    in n milliseconds. Setting the timeout
;                    variable from within the handler will
;                    cause a periodic timeout as shown in
;                    timeout 1.  (This is used to driver the
;                    Heartbeat LED.)
;---------------------------------------------------------------

t1_handler:

$IF   AA908   ;----------

       lda     iCalc_flag           ;is "interrupt Calc flag" on? (Set in Process_Dial during dial static times)
       beq     t1b                  ;no, continue
       jsr     Do_Calcs             ;yes, calculate/display SWR, Z, R, X (and L or C, if in those modes)
       lda     Mode_nr
       cmp     #Impedance_Mode      ;are we in impedance mode?
       bne     t1b                  ;no, just continue
       jsr     Disp_Sign            ;determine & display sign of reactance for Impedance Mode, during
                                    ;  dial "quiescent" times here in ISR
$ENDIF        ;-----------

t1b:   lda     #255
       sta     timeout1             ;reset timer 1 to 255 ms
       brclr   6,portF,t1a          ;and toggle heartbeat LED
       bclr    6,portF
       rts
t1a:   bset    6,portF
       rts

t2_handler:                         ;used for the Piezo beep duration
       nop
       rts

t3_handler:                         ;used for Power Down minutes counter
       ldhx    #$EA60               ;reload the 1ms counter to 60,000 (one minute)
       sthx    PwrDwnRegs
t3b:   lda     LiteOffMinCtr
       deca
       sta     LiteOffMinCtr
       bne     t3c
       jsr     TurnLiteOff          ;Backlite minutes counter timed out = turn off backlite
t3c:   rts

;**************************************************************************
;*****************       DELAY ROUTINES     *******************************
;**************************************************************************

;--------------------------------------------------------------------------
;   Delay -- This subroutine performs a simple software delay loop based upon
;            the value passed in ACC. The following timing calculation applies:
;               delay = ((ACC * 74) + 12) (tcyc)

delay:
        psha                           ;[2] save delay parameter temporarily
delay1:
        lda #22T                       ;[2] initialize 5us loop counter
                                       ;     (repeat for timing)
delay2:
        dbnza delay2                   ;[3] decrement inner delay loop counter
        dbnz 1,sp,delay1               ;[6] decrement outer delay loop counter
        pula                           ;[2] deallocate local variable
        rts                            ;[4] return

;-----------------------------------------------------
; Wait500ms -- Waits here for (10 x 50 x 1ms) = 500ms

wait500ms:
        lda     #10
        sta     count_hi
w500m1: lda     #$50
        sta     count_lo
w500m2: jsr     wait1ms
        dbnz    count_lo,w500m2
        dbnz    count_hi,w500m1
        rts

;-------------------------------------------------------
; Wait100ms-- Waits here for 100ms

wait100ms:
        lda     #100
w100a:  psha
        bsr     wait1ms
        pula
        dbnza   w100a
        rts

;--------------------------------------------------------
; Wait64ms-- Waits here for 64ms

wait64ms:
        lda     #64
w64a:   psha
        bsr     wait1ms
        pula
        dbnza   w64a
        rts

;---------------------------------------------------------
; Wait50ms-- Waits here for 50ms

wait50ms:
        mov     #50,timeout1
w50a:   tst     timeout1
        bne     w50a
        rts

;----------------------------------------------------------
; Wait32ms-- Waits here for 32ms

wait32ms:
        lda     #32
w32a:   psha
        bsr     wait1ms
        pula
        dbnza   w32a
        rts

;-----------------------------------------------------------
; Wait64ms-- Waits here for 16ms

wait16ms:
        lda     #16
w16a:   psha
        bsr     wait1ms
        pula
        dbnza   w16a
        rts

;------------------------------------------------------------
; Wait10ms-- Waits here for 10ms

Wait10ms:
        lda     #10
w10a:   psha
        bsr     wait1ms
        pula
        dbnza   w10a
        rts

;------------------------------------------------------------
; Wait1ms-- Waits here for 1ms

wait1ms:
        lda     #10
w1msa:  psha
        bsr     wait100us
        pula
        dbnza   w1msa
        rts

;-------------------------------------------------------------
; Wait100us -- Waits here for 100us
;              1 cycle = .11454545 US
;              (137 x (5 x .11454545)) + (6 x .11454545)
;              (174 x .5727) + .687 = 100 us

wait100us:
        lda     #174       ; [2]
w100u:  deca               ; [1]
        nop                ; [1]
        bne     w100u      ; [3]
        rts                ; [4]

;--------------------------------------------------------------
; Wait1us -- Waits here for 1us

wait1us:
        lda     us1
        dbnza   *
        rts

;**************************************************************
; DUMMY_ISR - Dummy Interrupt Service Routine.
;             Just does a return from interrupt.
;**************************************************************
dummy_isr:
       nop
       rti           ; return

;**************************************************************************
;***************      HCmon_LCD_disp        *******************************
;*     Routine called from HCmon to display message to LCD                *
;**************************************************************************

HCmon_LCD_disp:
        rts
